import { select, 
        csv, 
        scaleLinear, 
        max, 
        scaleBand, 
        axisLeft,
        axisBottom,
        format } from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => {
  const xValue = d => d.population;
  const yValue = d => d.country;
  
  const margin = { top: 50, left: 100, right: 50, bottom: 60};
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = scaleLinear()
  	.domain([0, max(data, xValue)])
  	.range([0, innerWidth]);
  // console.log(xScale.domain());
    
  const yScale = scaleBand()
  	.domain(data.map(yValue))
  	.range([0, innerHeight])
  	.padding(0.1);
  
  // const yAxis = axisLeft(yScale);

  // The whole thing is contained in group g, as we need to have identical margins. 
  const g = svg.append('g')
  	.attr('transform', `translate(${margin.left}, ${margin.top})`);
  
  // yAxis(g.append('g'));
  
  // A shortened version of appending a new group element that corresponds to the yAxis. Method "call" takes in the yAxis that we have just created.
  
  const xAxisTickFormat = number => 
  	format('.3s')(number)
  		.replace('G', 'B');
  
  const xAxis = axisBottom(xScale)
  	.tickFormat(xAxisTickFormat)
  	.tickSize(-innerHeight);
    
  // Here is the yAxis. 
  g.append('g')
    .call(axisLeft(yScale))
  	.selectAll('.domain, .tick line')
  		.remove();
  
  // Here is the xAxis. 
  const xAxisG = g.append('g').call(xAxis)
  	.attr('transform', `translate(0, ${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  xAxisG.append('text')
  	.attr('class', 'axis-label')
  	.attr('x', innerWidth/2)
  	.attr('y', 40)
  	.attr('fill', 'black')
  	.text('Population');
  
	g.selectAll('rect').data(data)
  	.enter().append('rect')
  		.attr('y', d => yScale(yValue(d)))
  		.attr('width', d => xScale(xValue(d)))
  		.attr('height', yScale.bandwidth());
  
  g.append('text')
  	.attr('class', 'title')
  	.attr('x', 240)
  	.attr('y', -10)
  	.text('Top 10 Most Populous Countries')
}

csv('data.csv').then(data => {
  data.forEach(d => {
  	d.population = +d.population * 1000;
  });
	render(data);
});